/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: Taxonomy.java,v e8a9fad0c4d6 2010/05/15 16:34:56 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.taxonomy;

import javax.annotation.Nonnull;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.io.Serializable;
import it.tidalwave.util.Id;
import it.tidalwave.util.NotFoundException;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class Taxonomy implements Serializable
  {
    private static final long serialVersionUID = 65968939768498676L;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public static enum Type implements Serializable
      {
        CLASS, ORDER, FAMILY, GENUS, SPECIES, SUBSPECIES;

        private static final long serialVersionUID = 6079865922121355L;

        @Nonnull
        public Type getNarrower()
          throws NotFoundException
          {
            final int narrowerIndex = this.ordinal() + 1;

            if (narrowerIndex >= values().length)
              {
                throw new NotFoundException("No narrower for " + this);
              }

            return values()[narrowerIndex];
          }
      }

    private final String displayName;

    protected final Map<Type, List<Taxon>> taxonListMapByType = new HashMap<Type, List<Taxon>>();

    protected final Map<Id, Taxon> taxonMapById = new HashMap<Id, Taxon>();
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public Taxonomy (final @Nonnull String displayName)
      {
        this.displayName = displayName;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public String getDisplayName()
      {
        return displayName;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Taxon findTaxonById (final @Nonnull Id id)
      throws NotFoundException
      {
        return NotFoundException.throwWhenNull(taxonMapById.get(id), "Can't find Taxon with id=" + id);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public List<Taxon> findTaxonsByType (final @Nonnull Type type)
      {
        return Collections.unmodifiableList(taxonListMapByType.get(type));
      }
  }
